@@ -30,6 +30,7 @@ data Cmd = Cmd
     ,cmdFiles :: Maybe [FilePath]    -- ^ which files to run it on, nothing = none given
     ,cmdHintFiles :: [FilePath]      -- ^ which settingsfiles to use
     ,cmdGivenHints :: [FilePath]     -- ^ which settignsfiles were explicitly given
+    ,cmdWithHints :: [String]        -- ^ hints that are given on the command line
     ,cmdReports :: [FilePath]        -- ^ where to generate reports
     ,cmdIgnore :: [String]           -- ^ the hints to ignore
     ,cmdShowAll :: Bool              -- ^ display all skipped items
@@ -41,6 +42,7 @@ data Cmd = Cmd
     ,cmdLanguage :: [Extension]      -- ^ the extensions (may be prefixed by "No")
     ,cmdQuiet :: Bool                -- ^ supress all console output
     ,cmdCross :: Bool                -- ^ work between source files, applies to hints such as duplicate code between modules
+    ,cmdProof :: [FilePath]          -- ^ a proof script to check against
     }
 
 
@@ -48,6 +50,7 @@ data Opts = Help
           | Ver
           | Test
           | Hints FilePath
+          | WithHint String
           | Path FilePath
           | Report FilePath
           | Skip String
@@ -61,6 +64,7 @@ data Opts = Help
           | Encoding String
           | FindHints FilePath
           | Language String
+          | Proof FilePath
           | Quiet
           | Cross
           | Ansi
@@ -71,6 +75,7 @@ opts = [Option "?" ["help"] (NoArg Help) "Display help message"
        ,Option "v" ["version"] (NoArg Ver) "Display version information"
        ,Option "r" ["report"] (OptArg (Report . fromMaybe "report.html") "file") "Generate a report in HTML"
        ,Option "h" ["hint"] (ReqArg Hints "file") "Hint/ignore file to use"
+       ,Option "w" ["with"] (ReqArg WithHint "hint") "Extra hints to use"
        ,Option "c" ["color","colour"] (NoArg Color) "Color output (requires ANSI terminal)"
        ,Option "i" ["ignore"] (ReqArg Skip "hint") "Ignore a particular hint"
        ,Option "s" ["show"] (NoArg ShowAll) "Show all ignored ideas"
@@ -84,6 +89,7 @@ opts = [Option "?" ["help"] (NoArg Help) "Display help message"
        ,Option "d" ["datadir"] (ReqArg DataDir "dir") "Override the data directory"
        ,Option "p" ["path"] (ReqArg Path "dir") "Directory in which to search for files"
        ,Option "q" ["quiet"] (NoArg Quiet) "Supress most console output"
+       ,Option ""  ["proof"] (ReqArg Proof "file") "Isabelle/HOLCF theory file"
        ,Option ""  ["cpp-define"] (ReqArg Define "name[=value]") "CPP #define"
        ,Option ""  ["cpp-include"] (ReqArg Include "dir") "CPP include path"
        ,Option ""  ["cpp-simple"] (NoArg SimpleCpp) "Use a simple CPP (strip # lines)"
@@ -115,7 +121,8 @@ getCmd args = do
     findHints <- concatMapM (getFile path exts2) [x | FindHints x <- opt]
 
     let hintFiles = [x | Hints x <- opt]
-    hints <- mapM (getHintFile dataDir) $ hintFiles ++ ["HLint" | null hintFiles]
+    let withHints = [x | WithHint x <- opt]
+    hints <- mapM (getHintFile dataDir) $ hintFiles ++ ["HLint" | null hintFiles && null withHints]
     let givenHints = if null hintFiles then [] else hints
 
     let languages = getExtensions [x | Language x <- opt]
@@ -136,6 +143,7 @@ getCmd args = do
         ,cmdFiles = files
         ,cmdHintFiles = hints
         ,cmdGivenHints = givenHints
+        ,cmdWithHints = withHints
         ,cmdReports = [x | Report x <- opt]
         ,cmdIgnore = [x | Skip x <- opt]
         ,cmdShowAll = ShowAll `elem` opt
@@ -147,6 +155,7 @@ getCmd args = do
         ,cmdLanguage = languages
         ,cmdQuiet = Quiet `elem` opt
         ,cmdCross = Cross `elem` opt
+        ,cmdProof = [x | Proof x <- opt]
         }
 
 
@@ -157,7 +166,7 @@ exitWithHelp = do
 
 
 versionText :: String
-versionText = "HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2011\n"
+versionText = "HLint v" ++ showVersion version ++ ", (C) Neil Mitchell 2006-2012\n"
 
 
 helpText :: String
