@@ -12,6 +12,7 @@ import Report
 import Idea
 import Apply
 import Test
+import Proof
 import Util
 import Parallel
 import HSE.All
@@ -49,6 +50,11 @@ hlint args = do
     let flags = parseFlags{cppFlags=cmdCpp, encoding=cmdEncoding, language=cmdLanguage}
     if cmdTest then
         test (\x -> hlint x >> return ()) cmdDataDir cmdGivenHints >> return []
+     else if notNull cmdProof then do
+        s <- readAllSettings cmd flags
+        let reps = if cmdReports == ["report.html"] then ["report.txt"] else cmdReports
+        mapM_ (proof reps s) cmdProof
+        return []
      else if isNothing cmdFiles && notNull cmdFindHints then
         mapM_ (\x -> putStrLn . fst =<< findSettings flags x) cmdFindHints >> return []
      else if isNothing cmdFiles then
@@ -58,14 +64,18 @@ hlint args = do
      else
         runHints cmd flags
 
-
-runHints :: Cmd -> ParseFlags -> IO [Suggestion]
-runHints Cmd{..} flags = do
-    let outStrLn x = unless cmdQuiet $ putStrLn x
-    settings1 <- readSettings cmdDataDir cmdHintFiles
+readAllSettings :: Cmd -> ParseFlags -> IO [Setting]
+readAllSettings Cmd{..} flags = do
+    settings1 <- readSettings cmdDataDir cmdHintFiles cmdWithHints
     settings2 <- concatMapM (fmap snd . findSettings flags) cmdFindHints
     settings3 <- return [Classify Ignore x ("","") | x <- cmdIgnore]
-    let settings = settings1 ++ settings2 ++ settings3
+    return $ settings1 ++ settings2 ++ settings3
+
+
+runHints :: Cmd -> ParseFlags -> IO [Suggestion]
+runHints cmd@Cmd{..} flags = do
+    let outStrLn x = unless cmdQuiet $ putStrLn x
+    settings <- readAllSettings cmd flags
 
     let files = fromMaybe [] cmdFiles
     ideas <- if cmdCross
