@@ -67,13 +67,16 @@ fromTyParen :: Type s -> Type s
 fromTyParen (TyParen _ x) = fromTyParen x
 fromTyParen x = x
 
--- is* :: Exp -> Bool
+-- is* :: Exp_ -> Bool
+-- is* :: Decl_ -> Bool
 isVar Var{} = True; isVar _ = False
+isCon Con{} = True; isCon _ = False
 isApp App{} = True; isApp _ = False
 isInfixApp InfixApp{} = True; isInfixApp _ = False
 isList List{} = True; isList _ = False
 isAnyApp x = isApp x || isInfixApp x
 isParen Paren{} = True; isParen _ = False
+isIf If{} = True; isIf _ = False
 isLambda Lambda{} = True; isLambda _ = False
 isMDo MDo{} = True; isMDo _ = False
 isBoxed Boxed{} = True; isBoxed _ = False
@@ -89,6 +92,7 @@ isPViewPat PViewPat{} = True; isPViewPat _ = False
 isParComp ParComp{} = True; isParComp _ = False
 isPatTypeSig PatTypeSig{} = True; isPatTypeSig _ = False
 isQuasiQuote QuasiQuote{} = True; isQuasiQuote _ = False
+isSpliceDecl SpliceDecl{} = True; isSpliceDecl _ = False
 
 isSection LeftSection{} = True
 isSection RightSection{} = True
