@@ -29,6 +29,8 @@ record field = Record{..}
 record = 1 --
 {-# LANGUAGE UnboxedTuples #-} \
 record = 1 --
+{-# LANGUAGE TemplateHaskell #-} \
+foo
 </TEST>
 -}
 
@@ -61,8 +63,8 @@ minimalExtensions x es = nub $ concatMap f es
 
 
 -- RecordWildCards implies DisambiguateRecordFields, but most people probably don't want it
-warnings old new | RecordWildCards `elem` old && RecordWildCards `notElem` new = "you may need to add DisambiguateRecordFields"
-warnings _ _ = ""
+warnings old new | RecordWildCards `elem` old && RecordWildCards `notElem` new = [Note "you may need to add DisambiguateRecordFields"]
+warnings _ _ = []
 
 
 used :: Extension -> Module_ -> Bool
@@ -76,7 +78,7 @@ used EmptyDataDecls = hasS f
           f _ = False
 used KindSignatures = hasT (un :: Kind S)
 used BangPatterns = hasS isPBangPat
-used TemplateHaskell = hasT2 (un :: (Bracket S, Splice S)) & hasS f
+used TemplateHaskell = hasT2 (un :: (Bracket S, Splice S)) & hasS f & hasS isSpliceDecl
     where f VarQuote{} = True
           f TypQuote{} = True
           f _ = False
