@@ -1,7 +1,7 @@
 {-# LANGUAGE PatternGuards, ViewPatterns #-}
 
 module Settings(
-    Severity(..), FuncName, Setting(..), isClassify, isMatchExp,
+    Severity(..), Note(..), showNotes, FuncName, Setting(..), isClassify, isMatchExp,
     defaultHintName, isUnifyVar,
     readSettings, readPragma, findSettings
     ) where
@@ -48,9 +48,31 @@ addInfix x = x{infixes = infix_ (-1) ["==>"] ++ infixes x}
 ---------------------------------------------------------------------
 -- TYPE
 
+data Note
+    = IncreasesLaziness
+    | DecreasesLaziness
+    | RemovesError String -- RemovesError "on []", RemovesError "when x is negative"
+    | ValidInstance String String -- ValidInstance "Eq" "x"
+    | Note String
+      deriving (Eq,Ord)
+
+instance Show Note where
+    show IncreasesLaziness = "increases laziness"
+    show DecreasesLaziness = "decreases laziness"
+    show (RemovesError x) = "removes error " ++ x
+    show (ValidInstance x y) = "requires a valid " ++ x ++ " instance for " ++ y
+    show (Note x) = x
+
+
+showNotes :: [Note] -> String
+showNotes = intercalate ", " . map show . filter use
+    where use ValidInstance{} = False -- Not important enough to tell an end user
+          use _ = True
+
+
 data Setting
     = Classify {severityS :: Severity, hintS :: String, funcS :: FuncName}
-    | MatchExp {severityS :: Severity, hintS :: String, scope :: Scope, lhs :: Exp_, rhs :: Exp_, side :: Maybe Exp_, notes :: String}
+    | MatchExp {severityS :: Severity, hintS :: String, scope :: Scope, lhs :: Exp_, rhs :: Exp_, side :: Maybe Exp_, notes :: [Note]}
     | Builtin String -- use a builtin hint set
     | Infix Fixity
       deriving Show
@@ -64,30 +86,31 @@ isMatchExp MatchExp{} = True; isMatchExp _ = False
 
 -- Given a list of hint files to start from
 -- Return the list of settings commands
-readSettings :: FilePath -> [FilePath] -> IO [Setting]
-readSettings dataDir xs = do
-    (builtin,mods) <- fmap unzipEither $ concatMapM (readHints dataDir) xs
+readSettings :: FilePath -> [FilePath] -> [String] -> IO [Setting]
+readSettings dataDir files hints = do
+    (builtin,mods) <- fmap unzipEither $ concatMapM (readHints dataDir) $ map Right files ++ map Left hints
     let f m = concatMap (readSetting $ moduleScope m) $ concatMap getEquations $ moduleDecls m
     return $ map Builtin builtin ++ concatMap f mods
 
 
 -- Read a hint file, and all hint files it imports
-readHints :: FilePath -> FilePath -> IO [Either String Module_]
+readHints :: FilePath -> Either String FilePath -> IO [Either String Module_]
 readHints dataDir file = do
-    y <- parseResult $ parseFile (addInfix parseFlags) file
+    let flags = addInfix parseFlags
+    y <- parseResult $ either (parseString flags "CommandLine") (parseFile flags) file
     ys <- concatM [f $ fromNamed $ importModule i | i <- moduleImports y, importPkg i `elem` [Just "hint", Just "hlint"]]
     return $ Right y:ys
     where
         f x | "HLint.Builtin." `isPrefixOf` x = return [Left $ drop 14 x]
-            | "HLint." `isPrefixOf` x = readHints dataDir $ dataDir </> drop 6 x <.> "hs"
-            | otherwise = readHints dataDir $ x <.> "hs"
+            | "HLint." `isPrefixOf` x = readHints dataDir $ Right $ dataDir </> drop 6 x <.> "hs"
+            | otherwise = readHints dataDir $ Right $ x <.> "hs"
 
 
 readSetting :: Scope -> Decl_ -> [Setting]
 readSetting s (FunBind _ [Match _ (Ident _ (getSeverity -> Just severity)) pats (UnGuardedRhs _ bod) bind])
     | InfixApp _ lhs op rhs <- bod, opExp op ~= "==>" =
         let (a,b) = readSide $ childrenBi bind in
-        [MatchExp severity (if null names then defaultHintName else head names) s (fromParen lhs) (fromParen rhs) a b]
+        [MatchExp severity (headDef defaultHintName names) s (fromParen lhs) (fromParen rhs) a b]
     | otherwise = [Classify severity n func | n <- names2, func <- readFuncs bod]
     where
         names = filter notNull $ getNames pats bod
@@ -115,15 +138,32 @@ readPragma o@(AnnPragma _ p) = f p
                     Nothing -> errorOn o "bad classify pragma"
                     Just severity -> Just $ Classify severity (ltrim b) ("",name)
             where (a,b) = break isSpace $ ltrim $ drop 6 s
+        g name (Paren _ x) = g name x
+        g name (ExpTypeSig _ x _) = g name x
+        g _ _ = Nothing
 readPragma _ = Nothing
 
 
-readSide :: [Decl_] -> (Maybe Exp_, String)
-readSide = foldl f (Nothing,"")
-    where f (Nothing,warn) (PatBind _ PWildCard{} Nothing (UnGuardedRhs _ side) Nothing) = (Just side, warn)
-          f (side,"") (PatBind _ (fromNamed -> "note") Nothing (UnGuardedRhs _ (Lit _ (String _ warn _))) Nothing) = (side,warn)
+readSide :: [Decl_] -> (Maybe Exp_, [Note])
+readSide = foldl f (Nothing,[])
+    where f (Nothing,notes) (PatBind _ PWildCard{} Nothing (UnGuardedRhs _ side) Nothing) = (Just side, notes)
+          f (side,[]) (PatBind _ (fromNamed -> "note") Nothing (UnGuardedRhs _ note) Nothing) = (side,g note)
           f _ x = errorOn x "bad side condition"
 
+          g (Lit _ (String _ x _)) = [Note x]
+          g (List _ xs) = concatMap g xs
+          g x = case fromApps x of
+              [con -> Just "IncreasesLaziness"] -> [IncreasesLaziness]
+              [con -> Just "DecreasesLaziness"] -> [DecreasesLaziness]
+              [con -> Just "RemovesError",str -> Just a] -> [RemovesError a]
+              [con -> Just "ValidInstance",str -> Just a,var -> Just b] -> [ValidInstance a b]
+              _ -> errorOn x "bad note"
+
+          con :: Exp_ -> Maybe String
+          con c@Con{} = Just $ prettyPrint c; con _ = Nothing
+          var c@Var{} = Just $ prettyPrint c; var _ = Nothing
+          str c = if isString c then Just $ fromString c else Nothing
+
 
 -- Note: Foo may be ("","Foo") or ("Foo",""), return both
 readFuncs :: Exp_ -> [FuncName]
