@@ -50,9 +50,9 @@ test main dataDir files = do
             res <- results $ mapM (testHintFile dataDir) files
             putStrLn ""
             return res
-    if failures == 0
-        then putStrLn $ "Tests passed (" ++ show total ++ ")"
-        else putStrLn $ "Tests failed (" ++ show failures ++ " of " ++ show total ++ ")"
+    putStrLn $ if failures == 0
+        then "Tests passed (" ++ show total ++ ")"
+        else "Tests failed (" ++ show failures ++ " of " ++ show total ++ ")"
     return failures
 
 
@@ -65,7 +65,7 @@ testHintFiles dataDir = do
 
 testHintFile :: FilePath -> FilePath -> IO Result
 testHintFile dataDir file = do
-    hints <- readSettings dataDir [file]
+    hints <- readSettings dataDir [file] []
     res <- results $ sequence $ nameCheckHints hints : checkAnnotations hints file :
                                 [typeCheckHints hints | takeFileName file /= "Test.hs"]
     progress
@@ -107,9 +107,14 @@ typeCheckHints hints = bracket
     where
         matches = filter isMatchExp hints
 
+        -- Hack around haskell98 not being compatible with base anymore
+        hackImport i@ImportDecl{importAs=Just a,importModule=b}
+            | prettyPrint b `elem` words "Maybe List Monad IO Char" = i{importAs=Just b,importModule=a}
+        hackImport i = i
+
         contents =
             ["{-# LANGUAGE NoMonomorphismRestriction, ExtendedDefaultRules #-}"] ++
-            concat (take 1 [map prettyPrint $ scopeImports $ scope x | x <- matches]) ++
+            concat [map (prettyPrint . hackImport) $ scopeImports $ scope x | x <- take 1 matches] ++
             ["main = return ()"
             ,"(==>) :: a -> a -> a; (==>) = undefined"
             ,"_noParen_ = id"
@@ -142,7 +147,7 @@ checkAnnotations setting file = do
             let good = case out of
                     Nothing -> null ideas
                     Just x -> length ideas == 1 &&
-                              length (show ideas) >= 0 && -- force, mainly for hpc
+                              seq (length (show ideas)) True && -- force, mainly for hpc
                               not (isParseError (head ideas)) &&
                               match x (head ideas)
             return $
@@ -219,10 +224,11 @@ checkInputOutput main xs = do
         Just got | length got == length want && and (zipWith eq want got) -> return pass
                  | otherwise -> do
             let trail = replicate (max (length got) (length want)) "<EOF>"
-            let (i,g,w):_ = [(i,g,w) | (i,g,w) <- zip3 [1..] (got++trail) (want++trail), not $ eq g w]
+            let (i,g,w):_ = [(i,g,w) | (i,g,w) <- zip3 [1..] (got++trail) (want++trail), not $ eq w g]
             putStrLn $ unlines
                 ["TEST FAILURE IN tests/" ++ pre
                 ,"DIFFER ON LINE: " ++ show i
                 ,"GOT : " ++ g
                 ,"WANT: " ++ w]
+            when (null want) $ putStrLn $ unlines $ "FULL OUTPUT FOR GOT:" : got
             return failure
